@@ -20,6 +20,7 @@ from .base import (
     NotFoundError,
     ParametersError,
     Service,
+    ServiceClosedError,
     _BaseProtocol,
     _BaseServerProtocol,
     )
@@ -143,6 +144,12 @@ class _ClientProtocol(_BaseProtocol):
             else:
                 call.set_result(answer)
 
+    def connection_lost(self, exc):
+        super().connection_lost(exc)
+        for call in self.calls.values():
+            if not call.cancelled():
+                call.cancel()
+
     def _translate_error(self, exc_type, exc_args, exc_repr):
         found = self.error_table.get(exc_type)
         if found is None:
@@ -158,6 +165,8 @@ class _ClientProtocol(_BaseProtocol):
                 self.counter)
 
     def call(self, name, args, kwargs):
+        if self.transport is None:
+            raise ServiceClosedError()
         bname = name.encode('utf-8')
         bargs = self.packer.packb(args)
         bkwargs = self.packer.packb(kwargs)
@@ -232,6 +241,7 @@ class _ServerProtocol(_BaseServerProtocol):
         else:
             if asyncio.iscoroutinefunction(func):
                 fut = asyncio.async(func(*args, **kwargs), loop=self.loop)
+                self.pending_waiters.add(fut)
             else:
                 fut = asyncio.Future(loop=self.loop)
                 try:
@@ -248,7 +258,10 @@ class _ServerProtocol(_BaseServerProtocol):
     def process_call_result(self, fut, *, req_id, peer, name,
                             args, kwargs,
                             return_annotation=None):
+        self.pending_waiters.discard(fut)
         self.try_log(fut, name, args, kwargs)
+        if self.transport is None:
+            return
         try:
             ret = fut.result()
             if return_annotation is not None:
@@ -256,6 +269,8 @@ class _ServerProtocol(_BaseServerProtocol):
             prefix = self.prefix + self.RESP_SUFFIX.pack(req_id,
                                                          time.time(), False)
             self.transport.write([peer, prefix, self.packer.packb(ret)])
+        except asyncio.CancelledError:
+            return
         except Exception as exc:
             prefix = self.prefix + self.RESP_SUFFIX.pack(req_id,
                                                          time.time(), True)
